import argparse
import logging

from importlib.metadata import version
from .commands import open_browser, iam_user_credentials, generate_okta_device_auth_credentials, init

def main():
    global_parser = argparse.ArgumentParser(add_help=False)
    global_parser.add_argument('-v', '--version', action='version', help="Display the version of this tool", version=version("aws_fusion"))
    global_parser.add_argument('--debug', action='store_true', help='Turn on debug logging')

    main_parser = argparse.ArgumentParser(prog='aws-fusion', description='Unified CLI tool for streamlined AWS operations, enhancing developer productivity', parents=[global_parser])
    subparsers = main_parser.add_subparsers(dest='command', required=True, help='Available commands')

    open_browser.setup(subparsers, global_parser)
    iam_user_credentials.setup(subparsers, global_parser)
    generate_okta_device_auth_credentials.setup(subparsers, global_parser)
    init.setup(subparsers, global_parser)

    args = main_parser.parse_args()
    
    if args.debug:
        logging.basicConfig(level=logging.DEBUG)
    
    # Call the associated function for the selected sub-command
    if hasattr(args, 'func'):
        args.func(args)


if __name__ == '__main__':
    main()



import sys
import boto3
import hashlib
import datetime
import logging
import json

from botocore.exceptions import ClientError
from botocore.utils import JSONFileCache

LOG = logging.getLogger(__name__)


class AssumeRoleWithSamlCache:
    __jsonFileCache = JSONFileCache()

    def __init__(self, role) -> None:
        LOG.debug('Initialize AssumeRoleWithSamlCache')
        self.__role = role
        self.__cache_key = hashlib.sha1(role.encode('utf-8')).hexdigest()
        self.__response = None

    def does_valid_token_cache_exists(self):
        if self.__cache_key in self.__jsonFileCache:
            response = self.__jsonFileCache[self.__cache_key]
            expiration = datetime.datetime.strptime(response['Credentials']['Expiration'], '%Y-%m-%dT%H:%M:%S%Z')
            current_utc_time = datetime.datetime.utcnow()
            if expiration - current_utc_time >= datetime.timedelta(minutes=1):
                response['Credentials']['Expiration'] = expiration.replace(tzinfo=datetime.timezone.utc)
                self.__response = response
                LOG.debug('Valid token exists. Can use cache')
                return True
        LOG.debug('No valid token exists')
        return False

    def credential_process(self):
        credentials = self.__response['Credentials']
        LOG.debug(f'Giving credential as aws credential process format')

        # https://docs.aws.amazon.com/cli/latest/userguide/cli-configure-sourcing-external.html
        return json.dumps({
            "Version": 1,
            "AccessKeyId": credentials['AccessKeyId'],
            "SecretAccessKey": credentials['SecretAccessKey'],
            "SessionToken": credentials['SessionToken']
        })
    
    def environment_variable(self):
        credentials = self.__response['Credentials']
        LOG.debug(f'Giving credential as environment variable format')

        command = '$env:' if sys.platform == 'win32' else 'export '

        return '\n'.join([
            f'{command}AWS_ACCESS_KEY_ID="{credentials["AccessKeyId"]}"',
            f'{command}AWS_SECRET_ACCESS_KEY="{credentials["SecretAccessKey"]}"',
            f'{command}AWS_SESSION_TOKEN="{credentials["SessionToken"]}"'
        ])

    def assume_role_with_saml(self, saml_response, roles, session_duration):
        LOG.debug(f'Started assuming role with SAML')
        client = boto3.Session(aws_access_key_id='dummy', aws_secret_access_key='dummy').client('sts')
        selected_role = self.__role
        try:
            response = client.assume_role_with_saml(
                RoleArn=selected_role,
                PrincipalArn=roles[selected_role],
                SAMLAssertion=saml_response,
                DurationSeconds=session_duration
            )
            LOG.debug('Got assume role response')
        except ClientError:
            # Try again with a shorter session length
            response = client.assume_role_with_saml(
                RoleArn=selected_role,
                PrincipalArn=roles[selected_role],
                SAMLAssertion=saml_response,
                DurationSeconds=3600
            )
            LOG.debug('Got assume role fallback response')

        self.__jsonFileCache[self.__cache_key] = response

        self.__response = response




import json

from urllib import parse, request

ISSUER = "aws-console-python-script"
SESSION_DURATION_IN_SECONDS = 43200


def signin_url(creds, region_name):
    login_request_url = __aws_login_url(
        creds.access_key, creds.secret_key, creds.token, region_name)
    return 'https://us-east-1.signin.aws.amazon.com/oauth?Action=logout' \
        f'&redirect_uri={parse.quote_plus(login_request_url)}'


def __aws_signin_token(access_key, secret_key, session_token) -> str:
    url_credentials = dict(sessionId=access_key,
                           sessionKey=secret_key, sessionToken=session_token)
    credentials_encoded = parse.quote_plus(json.dumps(url_credentials))
    request_url = 'https://signin.aws.amazon.com/federation?Action=getSigninToken' \
                  f'&Session={credentials_encoded}'  # f'&SessionDuration={SESSION_DURATION_IN_SECONDS}'

    with request.urlopen(request_url) as response:
        if not response.status == 200:
            raise Exception("Failed to get federation token")
        return json.loads(response.read())["SigninToken"]


def __aws_login_url(access_key, secret_key, session_token, region_name) -> str:
    destination_url_encoded = parse.quote_plus(
        "https://{}.console.aws.amazon.com/".format(region_name))
    signin_token_encoded = parse.quote_plus(
        __aws_signin_token(access_key, secret_key, session_token))
    return 'https://us-east-1.signin.aws.amazon.com/federation?Action=login' \
           f'&Issuer={ISSUER}' \
           f'&Destination={destination_url_encoded}' \
           f'&SigninToken={signin_token_encoded}'



import logging

from ..aws.assume_role import AssumeRoleWithSamlCache
from ..okta.api import device_auth, verification_and_token, session_and_token, saml_assertion

LOG = logging.getLogger(__name__)


def setup(subparsers, parent_parser):
    summary = 'Generate AWS session credentials using SAML assertion from Okta device authentication'
    parser = subparsers.add_parser('generate-okta-device-auth-credentials', description=summary, help=summary, parents=[parent_parser])

    parser.add_argument('--org-domain', required=True, help="Full domain hostname of the Okta org e.g. example.okta.com")
    parser.add_argument('--oidc-client-id', required=True, help="The ID is the identifier of the client is Okta app acting as the IdP for AWS")
    parser.add_argument('--aws-acct-fed-app-id', required=True, help="ID for the AWS Account Federation integration app")
    parser.add_argument('--aws-iam-role', required=True, help="The AWS IAM Role ARN to assume")
    parser.add_argument('--credential-process', action='store_true', help='Output the credential in AWS credential process syntax')

    parser.set_defaults(func=run)


def run(args):
    assume_role_with_cache = AssumeRoleWithSamlCache(args.aws_iam_role)

    if not assume_role_with_cache.does_valid_token_cache_exists():
        LOG.debug('Credential cache not found, invoking SAML')
        device_code = device_auth(args.org_domain, args.oidc_client_id)
        access_token, id_token = verification_and_token(args.org_domain, args.oidc_client_id, device_code)
        session_token = session_and_token(args.org_domain, args.oidc_client_id, access_token, id_token, args.aws_acct_fed_app_id)
        saml_response, roles, session_duration = saml_assertion(args.org_domain, session_token)
        assume_role_with_cache.assume_role_with_saml(saml_response, roles, session_duration)

    if args.credential_process:
        print(assume_role_with_cache.credential_process())
    else:
        print(assume_role_with_cache.environment_variable())



#!/usr/bin/env python
import re
import os
from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))


def read(*parts):
    return open(os.path.join(here, *parts), 'r').read()


def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(
        r"^__version__ = ['\"]([^'\"]*)['\"]", version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


setup(
    name='aws-fusion',
    version=find_version('aws_fusion', '__init__.py'),
    description='Unified CLI tool for streamlined AWS operations',
    keywords=[
        'aws',
        'aws-sdk',
        'aws-cli',
        'aws-authentication',
        'aws-sdk-python',
        'aws-auth'
    ],
    long_description=read("README.md"),
    long_description_content_type='text/markdown',
    packages=find_packages(),
    entry_points={
        'console_scripts': [
            'aws-fusion = aws_fusion.app:main',
        ]
    },
    install_requires=[
        'boto3>=1.29',
        'pyperclip>=1.8',
        'keyring>=24.3',
        'beautifulsoup4>=4.12',
        'requests>=2.31'
    ],
    author='Snigdhajyoti Ghosh',
    author_email='snigdhajyotighos.h@gmail.com',
    url='https://github.com/snigdhasjg/aws-fusion',
    license="MIT License",
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Operating System :: OS Independent',
        'Development Status :: 4 - Beta',
        'Topic :: Utilities'
    ],
)
